/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#ifndef __RQ_ASSERT__
#define __RQ_ASSERT__

// This all sets up the RqAssert() macro which is used like assert()
// but logs failed assertions via RqLog and provides a callback to do 
// any extra handling you would like.

#ifdef RQ_DEBUG
	// gnu c supports Pretty function which is very helpful, but nothing else
	// supports an equivalent that I know of.
	#ifdef __GNUC__
		#define RqAssert(condition) if(!(condition)) \
		_RqAssertFailed( __FILE__, __LINE__, __PRETTY_FUNCTION__, #condition )
	#else
		#define RqAssert(condition) if(!(condition)) \
		_RqAssertFailed( __FILE__, __LINE__, 0, #condition )
	#endif
#else
	#define RqAssert(condition)  // this turns off assertions on non-debug builds
#endif

// convenience function for when you are checking strings
#define RqAssertString(str1,str2) RqAssert( !RqStrCmp(str1,str2) )

// -- Failed assertions are handled by this.
void _RqAssertFailed(const char *filename, int lineno, const char *function, const char *condition);

// This is the type used for the assertion failure callback.
typedef void (*RqAssertCallback)(const char *filename, int lineno, const char *function, const char *condition);

// This allows you to handle assertions however you like,
// _RqAssertFailed will still log them first before calling
// the callback, but you can disable that by setting the log mask 
RqAssertCallback RqSetAssertCallback(RqAssertCallback inCallback);

#endif
