/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#ifndef __RQ_CONF__
#define __RQ_CONF__

// This library defines a platform identifier, attempts to determine big-endianness,
// and attempts to determine if this is a debug build or not.
// It also attempts to cause a compile error if it can't figure something out.

// It will define something like RQ_WINDOWS, RQ_MACOS, RQ_UNIX
// It treats Linux, Cygwin, and genuine unix variants as all being unix.

// Define a version constant incase we need to check it in a future version.
#define RQ_VERSION 0

// this is undefined when the platform is found.
#define RQ_UNKNOWN_PLATFORM 1

// -- Check for Macintosh platform.
#if defined MACOS || (defined(__MWERKS__) && defined(macintosh)) || defined(applec) || \
      defined(THINK_C) || defined(__SC__) || defined(TARGET_OS_MAC) || defined(macintosh)
#	define RQ_MACOS 1
#	define RQ_PLATFORM_NAME "MacOS"
#	undef RQ_UNKNOWN_PLATFORM
#endif


// -- Check for Windows platform.
// Exclude cygwin from windows so it will fall under unix case.
 // best windows check I've seen.
#if (defined(WINDOWS) || defined(_Windows) || defined(_WINDOWS) || \
       defined(WIN32) || defined(_WIN32) || defined(__WIN32__) \
	  ) && !defined(__CYGWIN__)
#	define RQ_WINDOWS 1
#	define RQ_PLATFORM_NAME "Windows"
#	undef RQ_UNKNOWN_PLATFORM
#endif

// -- Check for other platforms, meaning not unix, mac, or win that
// These aren't explicitly handled yet because I don't have
// anything setup to test them, but eventually I'll get around
// to it.`
#if defined(_AMIGA) || defined (__BEOS__) || defined(__MSDOS__) || defined(MSDOS) || defined(__OS2__)
#	define RQ_OTHER 1
#	define RQ_PLATFORM_NAME "Other"
#	undef RQ_UNKNOWN_PLATFORM
#endif

// -- Check for Unix or Unix-like platforms
#if defined(linux) || defined(BSD) || defined(unix) || defined(__unix__) \
	|| defined(__CYGWIN__) || (defined(POSIX) && !defined(RQ_UNKNOWN_PLATFORM))
#	define RQ_UNIX 1
#	define RQ_PLATFORM_NAME "Unix"
#	undef RQ_UNKNOWN_PLATFORM
#endif

// -- Cause a compiler error if we don't have a clue.
#if defined( RQ_UNKNOWN_PLATFORM )
#error "Rqal is unable to detect platform; proceed at your own risk or adjust rconf.h beforehand." 
#endif

// Visual C processor defines as of Visual C .net 2003
// _M_IX86 - x86 
// _M_IA64 - itanium
// _M_AMD64 - AMD 64 bit
// _M_MRX000 - mips
// _M_ALPHA - alpha
// _M_PPC - PPC
// _M_M68K - 68k

// NOTE: mips and alpha I think are big-endian, but until I can check I'm not messing with them.

// -- Try to determine endianness, i.e. native byte order.
// Macintosh no longer necessarily means big-endian, so we can't go by that 
// due to the new Intel based Mac's so we can't rely on that.
#if defined(__386__) || defined(i386) || defined(_M_IX86) || defined( _M_IA64 ) || defined(_M_AMD64)
// Visual C conveniently doesn't define these.
#	define RQ_LITTLE_ENDIAN
#elif defined( __ppc__ ) || defined( __ppc ) || defined(__PPCC__) || defined( __PPC__ ) || defined(_M_PPC) || defined(_M_M68K) || defined(powerc)
#	define RQ_BIG_ENDIAN
#else
#	error "Unable to determine endianness of platform."
#endif	



// -- Define for path seperator, what comes between parts of a file path.
#if defined( RQ_WINDOWS )
// NOTE: Mac is now / as well under MacOS X
// Need to figure out how to determine if under MacOS X or not.
#	define RQ_PATH_SEPERATOR '\\'
#elif defined( RQ_MACOS )
#	define RQ_PATH_SEPERATOR ':'
#else
#	define RQ_PATH_SEPERATOR '/'
#endif


// -- Byte swapping macros for your convenience.
#define RQ_SWAP16(x) \
     ((((x) >> 8) & 0xFF) | (((x) & 0xFF) << 8))
#define RQ_SWAP32(x) \
     ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) | \
      (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))


// -- Try to check whether this is a debug build or not.
#if defined( _DEBUG ) || defined( DEBUG )
#	define RQ_DEBUG
#endif

// -- Make sure NULL is defined because that is a pet peeve of mine.
#ifndef NULL
#define NULL 0
#endif

#endif
