/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#ifndef __RQ_ERROR__
#define __RQ_ERROR__

#ifdef __cplusplus
class RqError
{
	private:
		// This is the maximum length error message that will be stored.
		enum {
			MAX_ERROR_MESSAGE_LEN = 256
		};
		int mValue;
		int mLine;
		char mFile[MAX_ERROR_MESSAGE_LEN+1];
		char mMessage[MAX_ERROR_MESSAGE_LEN+1];

	public:
		enum {
			NO_ERR = 0,
			UNKNOWN,
			MEM_FULL,
			OPEN_ERR,
			CLOSE_ERR,
			EOF_ERR,
			DISK_FULL,
			PERM_ERR, // permissin denied
			PATH_ERR, // this is for dir as file, file as dir, file not found, dir not found
			READ_ERR,
			WRITE_ERR,
			TELL_ERR,
			SEEK_ERR,

			MAX_ERR // this is not a max error value but a place keeper
				// for the number of them.
		};

		RqError(int inErrVal);
		virtual ~RqError();

		// set additional associated error data.
		void SetErrorData(int line, const char *file, const char *message);
		
		// accessors
		int Value(void) const { return mValue; }
		int Line(void) const { return mLine; }
		const char *File(void) const { return mFile; }
		const char *Message(void) const { return mMessage; }

		// convenience function for throwing an error.
		static void Throw(int value, int line, const char *file, const char *message);	
};

// -- This macro constructs and throws an error along with line,file of origin and a message.
// If the log mask includes RqLog_ERROR then the error will also be logged.
#define RqThrow(err,msg) RqError::Throw(err,__LINE__,__FILE__,msg)
#endif

#endif
