/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#ifndef __RQ_FILE__
#define __RQ_FILE__

#ifndef __RQ_TYPES__
#include "rq_types.h"
#endif

#ifdef __cplusplus
#include <stdio.h>
// -- this is meant to abstract whatever you need to access a file, 
// The default implementation uses stdio since it's probably the most portable
// without dealing with the BS of iostream
typedef FILE *RqFileDesc;

// -- This is the value used for file positions and file lengths.
typedef long RqFilePos;

// -- This specifies where you are seeking relative to.
typedef enum {
	RqSeekPos=0,
	RqSeekStart,
	RqSeekEnd
} RqSeekFrom;

// The only state this class maintains is mDesc and the destructor does not close the file,
// so basically this class only acts as a convenient wrapper for the file routines.
class RqFile {
protected:
	RqFileDesc mDesc;
public:
	
	RqFile(void);
	virtual ~RqFile();

	// This allows you to get or set the file descriptor.
	RqFileDesc& Desc(void);

	// Open a file for reading or writing, we don't support both at once.
	void OpenForRead(const char *filename);
	// This will truncate the file if appendFlag is false, if true it will not
	// truncate, but position in file is undefined so call Seek
	void OpenForWriting(const char *filename, bool appendFlag);

	// Close the current file
	void Close(void);

	// Tell current position in file
	RqFilePos Tell(void);

	// Seek to a new position in the file
	void Seek(RqSeekFrom from, RqFilePos pos);

	// Read from the file, if EOF may read fewer bytes than requested or none,
	// Return's number of bytes read.
	RqFilePos Read(void *buf, RqFilePos len);

	// Write to file
	void Write(const void *buf, RqFilePos len);

	// Read a byte
	RqUns8 ReadByte(void);
	// Read a 16 bit value big endian
	RqUns16 ReadWordBE(void);
	// Read a 16 bit value little endian
	RqUns16 ReadWordLE(void);
	// Read a 32 bit value big endian
	RqUns32 ReadLongBE(void);
	// Read a 32 bit value little endian
	RqUns32 ReadLongLE(void);

	// Write a byte
	void WriteByte(RqUns8 inByte);
	// Write a 16 bit value big endian
	void WriteWordBE(RqUns16 inWord);
	// Write a 16 bit value little endian
	void WriteWordLE(RqUns16 inWord);
	// Write a 32 bit value big endian
	void WriteLongBE(RqUns32 inLong);
	// Write a 32 bit value little endian
	void WriteLongLE(RqUns32 inLong);

	// Utility functions for pathnames.

	// get filename portion of a full path, returns pointer into inpath
	static const char *GetFileNameFromFullPath(const char *inpath);

	// NOTE: write functions to get pathname from full path
	// NOTE: and also to get extension from filename.

};
#endif

#endif
