/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#ifndef __RQ_MEMORY__
#define __RQ_MEMORY__


// This module provides various memory debugging features.
// This header or rq.h must be preincluded by your project
// or compiler options for this to work as intended.
// Basically it keeps a list of all blocks allocated by new, new[], or RqAlloc,
// as well as NewPtr, NewPtrClear, NewHandle, and NewHandleClear on MacOS

#ifndef __RQ_TYPES__
#include "rq_types.h"
#endif

#if defined( RQ_DEBUG )&& !defined(_MT) && !defined(RQ_NO_MEM_DEBUG)
#define RQ_MEM_DEBUG
#endif

#ifdef __cplusplus
#include <cstdlib>
#include <cstring>

#define RQ_MEM_EXTERN extern "C"
#else
#include <stdlib.h>
#include <string.h>
// This is not present on all system, ifdef appropriately when it becomes a problem.
#include <malloc.h>

#define RQ_MEM_EXTERN
#endif

// NOTE: memory debug is not thread safe.
#if defined( RQ_MEM_DEBUG )
#define RqMemCopy(dst,src,size) _RqMemCopy(dst,src,size)
#define RqMemSet8(dst,val,size) _RqMemSet8(dst,val,size)
#define RqMemCompare(dst,src,size) _RqMemCompare(dst,src,size)
#define RqStrLen(str) _RqStrLen(str)
#define RqStrCmp(a,b) _RqStrCmp(a,b)
#define RqStrCopy(dst,src) _RqStrCopy(dst,src)
#define RqStrCat(dst,src) _RqStrCat(dst,src)
#define RqStrCaseNCmp(a,b,n) _RqStrCaseNCmp(a,b,n)
#define RqStrCaseCmp(a,b) _RqStrCaseCmp(a,b)
#else
// no RqMemInit if not debugging
#define RqMemInit()
#define RqMemDisableAccessWarnings()
#define RqMemSummary(comment)
#define RqMemDumpBlocks(comment)
#define RqMemCopy(dst,src,size) memcpy(dst,src,size)
#define RqMemSet8(dst,val,size) memset(dst,val,size)
#define RqMemCompare(dst,src,size) memcmp(dst,src,size)
#define RqStrLen(str) strlen(str)
#define RqStrCmp(a,b) strcmp(a,b)
#define RqStrCopy(dst,src) strcpy(dst,src)
#define RqStrCat(dst,src) strcat(dst,src)
#ifdef RQ_WINDOWS
		#define RqStrCaseNCmp(a,b,n) _strnicmp(a,b,n)
		#define RqStrCaseCmp(a,b) _stricmp(a,b)
	#else
		#define RqStrCaseNCmp(a,b,n) strncasecmp(a,b,n)
		#define RqStrCaseCmp(a,b) strcasecmp(a,b)
	#endif
#endif

#define RqMemZero(dst,size) RqMemSet8(dst,0,size)

RQ_MEM_EXTERN void _RqMemCopy(void *dst,const void *src,size_t size);
RQ_MEM_EXTERN void _RqMemSet8(void *dst,int val,size_t size);
RQ_MEM_EXTERN int _RqMemCompare(const void *dst,const void *src,size_t size);

RQ_MEM_EXTERN void _RqMemSet16(void *dst,RqUns16 val,size_t inCount);
RQ_MEM_EXTERN void _RqMemSet32(void *dst,RqUns32 val,size_t inCount);

RQ_MEM_EXTERN long _RqStrLen(const char *str);

RQ_MEM_EXTERN int _RqStrCmp(const char *inA, const char *inB);
RQ_MEM_EXTERN void _RqStrCopy(char *outStr, const char *inStr);
RQ_MEM_EXTERN void _RqStrCat( char *dst, const char *src );
RQ_MEM_EXTERN RqInt32 _RqStrCaseCmp( const char *a, const char *b );
RQ_MEM_EXTERN RqInt32 _RqStrCaseNCmp( const char *a, const char *b, RqUns32 n );


// This one has no corresponding equivalent in stdlib so it's not a macro
// same with this one
RQ_MEM_EXTERN void RqMemSet16(void *dst,RqUns16 val,size_t inCount);
RQ_MEM_EXTERN void RqMemSet32(void *dst,RqUns32 val,size_t inCount);

// Use this to label where new and new[] operator allocations took place
// since we can't use macros to pass in file/line information for those.
// just call it before new is called again.
#if defined( RQ_MEM_DEBUG )
#define RqLabelNew(ptr) _RqLabelNew(ptr,__FILE__,__LINE__)
#else
#define RqLabelNew(ptr)
#endif

// ATTN: Use this to verify memory (other than handles on MacOS) are allocated,
// by passing it a pointer and a size. It will log the failure and assert if not.
// RqCheckMem provides one of the main purposes of this entire memory checking module.
// RqDeclareMem declares statically declared objects so that RqCheckMem doesn't
// think memory in them is bogus.
#if defined( RQ_MEM_DEBUG )
#define RqDeclareMem(ptr,size) _RqDeclareMem((void *)(ptr),(unsigned long)(size),__FILE__,__LINE__)
#define RqUndeclareMem(ptr,size) _RqUndeclareMem((void *)(ptr),__FILE__,__LINE__)
#define RqCheckMem(ptr,size) _RqCheckMem((void *)(ptr),(unsigned long)(size),__FILE__,__LINE__)
#else
#define RqDeclareMem(ptr,size)
#define RqUndeclareMem(ptr,size)
#define RqCheckMem(ptr,size)
#endif

#if defined(RQ_MEM_DEBUG)

// Mac specific stuff.
#ifdef RQ_MACOS
#include <Memory.h>
#define NewPtr(x)			(Ptr)RqMemAllocPointer( (x),__FILE__,__LINE__,kBlockPointer )
#define NewHandle(x)		RqMemAllocHandle( (x),__FILE__,__LINE__ )
#define DisposePtr(x)		RqMemFreePointer( (void *)(x), __FILE__,__LINE__ )
#define DisposeHandle(x)	RqMemFreeHandle( (x),__FILE__,__LINE__ )
#define NewPtrClear(x)		(Ptr)RqMemAllocPointerClear( (x),__FILE__,__LINE__,kBlockPointer )
#define NewHandleClear(x)	RqMemAllocHandleClear( (x),__FILE__,__LINE__ )
#endif

#define RqAlloc(x)			RqMemAllocPointer( (x),__FILE__,__LINE__,kBlockPointer )
#define RqFree(x)			RqMemFreePointer( (x),__FILE__,__LINE__ )

// ATTN: We are going to try defining these, and seeing if it works well enough,
// if it causes you problems, define RQ_NO_MALLOC in your compile options.
#ifndef RQ_NO_MALLOC
#define malloc(x)			RqMemAllocPointer( (x),__FILE__,__LINE__,kBlockPointer )
#define free(x)			RqMemFreePointer( (x),__FILE__,__LINE__ )
#endif


typedef enum DebugBlockType_t
	{
	kBlockPointer, // malloc or NewPtr
	kBlockHandle, // NewHandle on mac
	kBlockNew, // new[] or new
	kBlockDeclared // declared explicitly(for staticly allocated, locked handles,etc)
	} DebugBlockType;


// this function must be called at startup from main()
// so that values are properly initialized 
// for rq_memory
RQ_MEM_EXTERN void RqMemInit(void);
// turn off warnings for bad memory accesses, often caused by accessing constants
// until we fix this to work better.
RQ_MEM_EXTERN void RqMemDisableAccessWarnings( void );

RQ_MEM_EXTERN void RqMemReset( void );
RQ_MEM_EXTERN void RqMemSummary( const char *inComment );
RQ_MEM_EXTERN void RqMemDumpBlocks(const char *inComment);
RQ_MEM_EXTERN void * RqMemAllocPointer( unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine,DebugBlockType inType );
RQ_MEM_EXTERN void RqMemFreePointer( void * refconst, char *inSrcFile,unsigned long inSrcLine );
RQ_MEM_EXTERN void * RqMemAllocPointerClear( unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine,DebugBlockType inType );
#ifdef __cplusplus
void _RqLabelNew(void *ref,char *inSrcFile,unsigned long inSrcLine);
#endif
RQ_MEM_EXTERN void _RqCheckMem(void *ptr,unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine);
RQ_MEM_EXTERN void _RqDeclareMem(void *ptr,unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine);
RQ_MEM_EXTERN void _RqUndeclareMem( void *ptr, char *inSrcFile,unsigned long inSrcLine );
// This checks that a block of memory specified by pointer/size is allocated, 
// it doesn't work for handles.
// ATTN: This is only public so unit test can verify it works.
RQ_MEM_EXTERN int _RqCheckMemWork(void *ptr,unsigned long inSize);


#ifdef RQ_MACOS
RQ_MEM_EXTERN Handle RqMemAllocHandle( unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine );
RQ_MEM_EXTERN void RqMemFreeHandle( Handle refconst, const char *inSrcFile,unsigned long inSrcLine );
RQ_MEM_EXTERN Handle RqMemAllocHandleClear( unsigned long inSize,const char *inSrcFile,unsigned long inSrcLine );
#endif

#endif // __BX_DEBUG__

#endif // __RQ_MEMORY__
