/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2007, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

 ////////                 
   //    //    ////////    This file is part of rqal, a.k.a. _Richard's Quality Library_ .
   //    //   //    //     Visit http://www.ranton.org for more information.
   //////     //    //     Email ranton@ranton.org for support or licensing.  
   //   //     //// //     
   //    //          //    Copyright (c) 2007 Richard Neal Anton
   //    //          //    All Rights Reserved.
  ///    //           //

#include <iostream>
#include <sstream>
using namespace std;

#ifndef __RQ__
#include "rq.h"
#endif


bool CheckConf(void);
bool TestLog(void);
bool TestLogStream(void);
bool TestAssert(void);
bool TestError(void);
bool TestFile(void);
bool TestMemory(void);

void MyAssertHandler(const char *filename, int lineno, const char *function, const char *condition);


int main(int argc, char **argv)
{
	RqMemInit();

	bool bad = false;

	// Run all unit tests.
	if( !RqUnitTest("Conf",CheckConf) )
		bad = true;
	if( !RqUnitTest("Log",TestLog) )
		bad = true;
	if( !RqUnitTest("LogStream",TestLogStream) )
		bad = true;
	if( !RqUnitTest("Assert",TestAssert) )
		bad = true;
	if( !RqUnitTest("Error",TestError) )
		bad = true;
	if( !RqUnitTest("File",TestFile) )
		bad = true;
	if( !RqUnitTest("Memory",TestMemory) )
		bad = true;

	// Return indicator value to operating system for scripts.
	return bad ? 1 : 0;
}


bool CheckConf(void)
{
	// This test is windows specific, so we will just pass
	// for other platforms until they update it.
	// Some of these tests are actually determined at compile time for this one
	// since we are testing preprocessor crap
#ifdef WIN32
	if( RQ_PATH_SEPERATOR != '\\' )
		return false;
	if( strcmp(RQ_PLATFORM_NAME,"Windows") )
		return false;
#if !defined(RQ_LITTLE_ENDIAN)
	return false;
#endif
#if defined(RQ_BIG_ENDIAN)
	return false;
#endif
#if defined(_DEBUG) && !defined(RQ_DEBUG)
	return false;
#endif
#if !defined(_DEBUG) && defined(RQ_DEBUG)
	return false;
#endif

#elif defined __GNUC__
	// we are on some kind of unix/linux something probably,
	// or atleast under GCC
	//
#else
	return true; // not writing this yet.
#endif

	// passed if we made it this far.
	return true;
}

// Test logging procedures.
bool TestLog(void)
{
// Forgive the lack of indentation, makes the string constant a lot easier.
const char *kExpected="\
error: You should see this.\n\
trace: log trace\n\
dbg: log debug\n\
info: log info\n\
mem: log memory\n\
warning: log warning\n\
error: log error\n\
assert: log assert\n\
log app1\n\
";

	// Use a stringstream to test that the log output as it should.
	stringstream sstr;

	RqSetLogCallback(RqStreamLogCallback,&sstr);

	RqSetLogMask(RqLog_ERROR);

	// log something allowed by our mask.
	RqLog(RqLog_ERROR,"You should see this.");

	// log something not allowed by default mask
	RqLog(RqLog_TRACE,"You should not see this.");

	// set log mask to allow everything
	RqLogType oldMask = RqSetLogMask(RqLog_ALL);
	
	// log every type to make sure they display correctly.
	RqLog(RqLog_TRACE,"log trace");
	RqLog(RqLog_DEBUG,"log debug");
	RqLog(RqLog_INFO,"log info");
	RqLog(RqLog_MEMORY,"log memory");
	RqLog(RqLog_WARNING,"log warning");
	RqLog(RqLog_ERROR,"log error");
	RqLog(RqLog_ASSERT,"log assert");

	// log a custom log level message.
	RqLog(RqLog_APP1,"log app1");

	// restore mask value.
	(void)RqSetLogMask(oldMask);

	
	// put it back to the default callback.
	RqSetLogCallback(RqDefaultLogCallback,NULL);

	// now check that the contents of sstr are correct.
	if(strcmp(sstr.str().c_str(),kExpected))
		return false;

	return true; // NOTE: fix this.
}


bool TestLogStream(void)
{
	const char *kCorrectOutput = "info: hey bitch\n\
\n\
info: first second\n\
info: temp in place\n\
warning: be warned\n\
dbg: bugnum is -111\n\
dbg: unsigned 15: 15\n\
dbg: bignum is 1000000001\n\
dbg: neg bignum is -1000000001\n\
error: ubig: 4294967295\n\
info: bye now\n";

	RqLogCallback oldCallback = RqSetLogCallback(RqLogCaptureCallback,NULL);

	RqLogType oldMask = RqSetLogMask(RqLog_ALL);

	RqInfo( "hey bitch\n" );

	rqlog(RqLog_INFO) << "first " << "second";
	rqinfo << "temp in place";
	
	rqwarning << "be warned";

	int bugnum = -111;
	rqdebug << "bugnum is " << bugnum;

	unsigned uval = 15;
	rqdebug << "unsigned 15: " << uval;

	long bignum = 1000000001;
	rqdebug << "bignum is " << bignum;
	rqdebug << "neg bignum is " << -bignum;

	unsigned long ubig = 0xffffffff;
	rqerror << "ubig: " << ubig;

	RqInfo( "bye now" );

	const char *captured = RqLogCapture();

	RqSetLogCallback( oldCallback, NULL );
	RqSetLogMask(oldMask);
	if( RqStrCmp(kCorrectOutput, captured) )
		return false;

	return true;
}


static bool sAssertedFlag;
bool TestAssert(void)
{
	bool dbg = false;
#ifdef RQ_DEBUG
	dbg = true;
#endif

	// just assert something false.
	int x = -1;
	sAssertedFlag = false;
	
	// quiet the normal output.
	RqLogType oldMask = RqSetLogMask(RqLog_NONE);
	RqSetAssertCallback(MyAssertHandler);
	RqAssert( x > 0 );
	RqSetLogMask(oldMask);

	// make sure they are off on release and working on debug
	return sAssertedFlag == dbg; // true;
}

void MyAssertHandler(const char *filename, int lineno, const char *function, const char *condition)
{
	sAssertedFlag = true;
}

bool TestError(void)
{
	bool flag = false;
	// quiet the normal output.
	RqLogType oldMask = RqSetLogMask(RqLog_NONE);
	
	try
	{
		RqThrow(RqError::MEM_FULL,"This is a test error.");
	}
	catch(RqError& err)
	{
		// cout << "Caught: #" << err.Value() << " " << err.Message() << " " << err.File() << ":" << err.Line() << "\n";
		if( err.Value() == RqError::MEM_FULL )
			flag = true;
	}
	RqSetLogMask(oldMask);

	return flag;
}

bool TestFile(void)
{
	RqFile file;
	const char *filename = "testfile.txt";
	RqFilePos length;
	char *contents=0;

	try
	{
		#define line1 "Line 1\n"
		#define line2 "line 2\n"
		#define bothLines "Line 1\nline 2\n"

		// open a file for writing and let it be truncated.
		file.OpenForWriting(filename,false);
		// write something to the file
		file.Write((const void *)line1,sizeof(line1)-1);
		// close the file
		file.Close();
		// now open the file to be appended,
		file.OpenForWriting(filename,true);
		// seek to the end
		file.Seek(RqSeekEnd,0);
		// write something to it
		file.Write(line2,sizeof(line2)-1);
		// close the file
		file.Close();
		// open the file for reading
		file.OpenForRead(filename);
		// seek to end
		file.Seek(RqSeekEnd,0);
		// do tell to get file length
		length = file.Tell();
		if( length != sizeof(line1)+sizeof(line2)-2 )
			RqThrow(RqError::UNKNOWN,"File length incorrect");
		// seek to beginning
		file.Seek(RqSeekStart,0);
		// read in the entire file 
		contents = new char[length+1];
		if( !contents )
			RqThrow(RqError::MEM_FULL,"Not enough memory");
		if( file.Read((void *)contents,length) != length )
			RqThrow(RqError::EOF_ERR,"Unexpected end of file");
		// print it out
		// cout.write(contents,length);
		contents[length] = 0;
		if( strcmp(contents,bothLines) )
			RqThrow(RqError::UNKNOWN,"File contents do not match");
		delete[] contents;
		contents = 0;
		// close the file
		file.Close();
	}
	catch(RqError& err)
	{
		if( contents )
			delete[] contents;
		cout << "FileTest caught: #" << err.Value() << " " << err.Message() << " " << err.File() << ":" << err.Line() << "\n";
		try {
			file.Close(); // may not be open, but that's ok.
		} catch(...) {}

		return false;
	}
	return true;
}



const char *kMemCheckString = "2 blocks detected, totalling 111 bytes";

bool TestMemory(void)
{
#ifndef RQ_MEM_DEBUG
	cout << "Memory debugging disabled\n";
	return true;
#ifdef RQ_DEBUG
	cout << "Debugging is enabled\n";
#endif
#ifdef MT
	cout << "MT is defiend\n";
#endif
	return true;
#else

	RqLogCaptureReset();

	RqSetLogCallback(RqLogCaptureCallback,NULL);
	RqLogType oldMask = RqSetLogMask(RqLog_MEMORY);
	
	static char staticArray[50];
	RqDeclareMem(staticArray,50);

	// allocate some block so that we know we have something outstanding.
	char stackArray[100];
	char *ptr = new char[69];
	RqLabelNew((void *)ptr);
	char *ptr2 = (char *)RqAlloc(sizeof(char)*42);
	
#ifdef RQ_DEBUG

	// check that these check out goo.d
	if( !_RqCheckMemWork(ptr2,42) )
		return false;
	if( !_RqCheckMemWork(ptr,69) )
		return false;
	if( !_RqCheckMemWork(ptr+30,30) )
		return false;
	// check that stuff on stack works.
	if( !_RqCheckMemWork(stackArray,100) )
		return false;
	// check that statically declared stuff works.
	if( !_RqCheckMemWork(staticArray,50) )
		return false;

	RqMemDumpBlocks("TestMemory");
#endif
	RqFree(ptr2);
	delete[] ptr;

	// check that these are recognized as bad now
	if( _RqCheckMemWork(ptr2,42) )
		return false;
	if( _RqCheckMemWork(ptr,69) )
		return false;
	if( _RqCheckMemWork(ptr2+20,22) )
		return false;

	// put it back to the default callback.
	RqSetLogCallback(RqDefaultLogCallback,NULL);
	RqSetLogMask(oldMask);

#ifdef RQ_DEBUG
	// now check that the contents of sstr are correct.
	const char *capture = RqLogCapture();
	// as long as we can find this string in the output, it's probably working right.
	if(strstr(capture,kMemCheckString) == NULL)
	{
		cout.write(capture,RqStrLen(capture));
		return false;
	}
#endif

	return true;
#endif // RQ_MEM_DEBUG
}
